@@ -9,7 +9,7 @@ require Exporter;
 
 @ISA = qw(Exporter);
 @EXPORT_OK = qw(updatedir);
-$VERSION = sprintf "%d.%03d", q$Revision: 1.14 $ =~ /(\d+)\.(\d+)/;
+$VERSION = sprintf "%d.%03d", q$Revision: 1.15 $ =~ /(\d+)\.(\d+)/;
 $CAUTION ||= 0;
 $TRY_SHORTNAME ||= 0;
 $SIGNING_PROGRAM ||= 'gpg --clearsign --default-key ';
@@ -135,7 +135,7 @@ sub updatedir ($) {
       $cksum .= $_;
     }
     close $fh;
-    if ( !$SIGNING_KEY ^ !!$is_signed ) { # either both or neither
+    if ( !!$SIGNING_KEY == !!$is_signed ) { # either both or neither
       return 1 if $cksum eq $ddump;
       return 1 if ckcmp($cksum,$dref);
     }
@@ -157,9 +157,17 @@ sub updatedir ($) {
 
   printf $fh "# CHECKSUMS file written on %s by CPAN::Checksums (v%s)\n%s",
       scalar gmtime, $VERSION, $ddump;
-  close $fh;
+  my $success = close $fh;
+  if ($SIGNING_KEY && !$success) {
+    warn "Couldn't run '$SIGNING_PROGRAM $SIGNING_KEY'!
+Falling back to ordinary append mode.";
+    open $fh, ">>$ckfn\0" or die "Couldn't open >>$ckfn\: $!";
+    printf $fh "# CHECKSUMS file written on %s by CPAN::Checksums (v%s)\n%s",
+        scalar gmtime, $VERSION, $ddump;
+    close $fh or warn "Couldn't close $ckfn in append mode: $!";
+  }
   chmod 0444, $ckfn or die "Couldn't chmod to 0444 for $ckfn\: $!";
-  2;
+  return 2;
 }
 
 sub ckcmp ($$) {
